Introduction
===

AppDynamics’ Node.js agent allows you to monitor your Node.js applications in production so you get real-time visibility of server health, application performance, and resource metrics.

The causes of performance problems can be varied and complex to identify while the results usually boil down to a slow or unavailable web site. The AppDynamics Node.js agent reveals the internals of your application and infrastructure through Node.js process-level visibility and proactive monitoring using policies, enabling detailed analysis, fast troubleshooting, performance and capacity optimization. The agent’s process snapshots let you examine your code execution at a multi-process level so you can find the root cause of slow performance in minutes.


[Read our support matrix](https://docs.appdynamics.com/display/PRO44/Node.js+Supported+Environments) for details on the supported Node.js versions, operating systems, and backends. The supported environments are constantly being updated.

Requirements
--
To use the AppDynamics Node.js agent, you need the following:

-   An AppDynamics account with a running controller. [Sign up for an account here](https://portal.appdynamics.com/account/).
-   You will receive an email with your controller *host* and *port*.
- To download and install AppDynamics on your own, [download the controller from here](http://download.appdynamics.com/browse/zone/3/) and then [follow these instructions on how to install](https://docs.appdynamics.com/display/PRO44/Install+the+Controller).


Installing the Node.js agent
--
AppDynamics requires that the agent major version is <= the controller major version.  The latest version of the agent guaranteed to work with any SaaS controller is indicated on this page. You can install it by typing the following command from the root directory of the Node.js application that you are instrumenting:

```sh
npm install appdynamics
```

If you know you want the latest published agent version, you can use the next tag:
```sh
npm install appdynamics@next
```

If you know which specific version of the Node.js agent you want to install, you can specify it:

```sh
npm install appdynamics@<version>
```
Then paste the following in your application as the very first line of your application source code, before any other require statement:  

```sh
require('appdynamics').profile({
  controllerHostName: '<controller host name>',
  controllerPort: <controller port number>,
  controllerSslEnabled: false,  // Set to true if controllerPort is SSL
  accountName: '<AppDynamics_account_name>',
  accountAccessKey: '<AppDynamics_account_key>',
  applicationName: 'your_app_name',
  tierName: 'choose_a_tier_name',
  nodeName: 'choose_a_node_name'
 });
```

[See here for further installation details](https://docs.appdynamics.com/display/PRO44/Install+the+Node.js+Agent).

For More Information
--
[Overview of Monitoring and Troubleshooting with AppDynamics](https://docs.appdynamics.com/display/PRO44/APM+for+Node.js)
.
